@@ -13,13 +13,28 @@ class DataCube(Array):
     Stores 4D-STEM datasets.
     """
 
+    # import class methods
     from py4DSTEM.io.datastructure.py4dstem.datacube_fns import (
         add,
-        get_diffraction_image,
+        set_scan_shape,
+        swap_RQ,
+        swap_Rxy,
+        swap_Qxy,
+        crop_Q,
+        crop_R,
+        bin_Q,
+        bin_Q_mmap,
+        bin_R,
+        thin_R,
+        pad_Q,
+        resample_Q,
+        filter_hot_pixels,
+        get_virtual_diffraction,
         get_dp_max,
         get_dp_mean,
         get_dp_median,
         get_virtual_image,
+        position_detector,
         get_vacuum_probe,
         find_Bragg_disks,
         get_probe_size,
