@@ -1,7 +1,7 @@
 # Functions to become DataCube methods
 
 import numpy as np
-from scipy.ndimage import distance_transform_edt, binary_fill_holes
+from scipy.ndimage import distance_transform_edt, binary_fill_holes, gaussian_filter
 
 
 # Add to tree
@@ -924,8 +924,6 @@ def find_Bragg_disks(
     ml_num_attempts = 1, 
     ml_batch_size = 8,
    
-    _qt_progress_bar = None,
-
     name = 'braggvectors',
     returncalc = True,
     ):
@@ -1025,8 +1023,6 @@ def find_Bragg_disks(
                     processing
             if distributed is None, which is the default, processing will be in
             serial
-        _qt_progress_bar (QProgressBar instance): used only by the GUI for serial
-            execution
         name (str): name for the output BraggVectors
         returncalc (bool): if True, returns the answer
 
@@ -1074,8 +1070,6 @@ def find_Bragg_disks(
         ml_model_path = ml_model_path, 
         ml_num_attempts = ml_num_attempts, 
         ml_batch_size = ml_batch_size,
-
-        _qt_progress_bar = _qt_progress_bar,
     )
 
 
@@ -1100,8 +1094,11 @@ def find_Bragg_disks(
 def get_beamstop_mask(
     self,
     threshold = 0.25,
-    distance_edge = 4.0,
+    distance_edge = 2.0,
     include_edges = True,
+    sigma = 0,
+    use_max_dp = False,
+    scale_radial = None,
     name = "mask_beamstop",
     returncalc = True,
     ):
@@ -1114,6 +1111,10 @@ def get_beamstop_mask(
             pixel, while 1 uses the brighted pixels.
         distance_edge: (float)  How many pixels to expand the mask.
         include_edges: (bool)   If set to True, edge pixels will be included in the mask.
+        sigma: (float)          Blurring to apply in pixels before thresholding.
+        use_max_dp: (bool)      Set to True to use max diffraction pattern instead of beam.
+        scale_radial: (float)   Scale image intensities from distance from image center * this value.
+                                This can help when the center spot is extremely bright under the beam stop.
         name: (string)          Name of the output array.
         returncalc: (bool):     Set to true to return the result.
 
@@ -1122,19 +1123,37 @@ def get_beamstop_mask(
 
     """
 
-    # Calculate dp_mean if needed
-    if not "dp_mean" in self.tree.keys():
-        self.get_dp_mean();
+    if scale_radial is not None:
+        x = np.arange(self.data.shape[2]) * 2.0 / self.data.shape[2]
+        y = np.arange(self.data.shape[3]) * 2.0 / self.data.shape[3]
+        ya, xa = np.meshgrid(y - np.mean(y), x - np.mean(x))
+        im_scale = 1.0 + np.sqrt(xa**2 + ya**2)*scale_radial
 
-    # normalized dp_mean
-    int_sort = np.sort(self.tree["dp_mean"].data.ravel())
+    # Get image for beamstop mask
+    if use_max_dp:
+        if not "dp_mean" in self.tree.keys():
+            self.get_dp_max();
+        im = self.tree["dp_max"].data.astype('float')
+
+    else:
+        if not "dp_mean" in self.tree.keys():
+            self.get_dp_mean();
+        im = self.tree["dp_mean"].data.astype('float')
+
+    # smooth and scale if needed
+    if sigma > 0.0:
+        im = gaussian_filter(im, sigma, mode='nearest')
+    if scale_radial is not None:
+        im *= im_scale
+
+    # Calcualte beamstop mask
+    int_sort = np.sort(im.ravel())
     ind = np.round(np.clip(
             int_sort.shape[0]*threshold,
             0,int_sort.shape[0])).astype('int')
     intensity_threshold = int_sort[ind]
+    mask_beamstop = im >= intensity_threshold
 
-    # Use threshold to calculate initial mask
-    mask_beamstop = self.tree["dp_mean"].data >= intensity_threshold
 
     # clean up mask
     mask_beamstop = np.logical_not(binary_fill_holes(np.logical_not(mask_beamstop)))
@@ -1147,9 +1166,8 @@ def get_beamstop_mask(
         mask_beamstop[-1,:] = False
         mask_beamstop[:,-1] = False
 
-
     # Expand mask
-    mask_beamstop = distance_transform_edt(mask_beamstop) < distance_edge
+    mask_beamstop = distance_transform_edt(mask_beamstop) < distance_edge + 0.1
 
     # Output mask for beamstop
     self.name = name
