@@ -2,6 +2,7 @@
 import numpy as np
 import dask.array as da
 from py4DSTEM.utils.tqdmnd import tqdmnd
+from py4DSTEM.io.datastructure.py4dstem import Calibration
 
 def get_virtual_image(
     datacube,
@@ -12,7 +13,8 @@ def get_virtual_image(
     shift_center = False,
     verbose = True,
     dask = False,
-    return_mask = False
+    return_mask = False,
+    test_config = False
 ):
     '''
     Function to calculate virtual image
@@ -27,83 +29,68 @@ def get_virtual_image(
                 - 'annular' or 'annulus' uses annular detector, like dark field
                 - 'rectangle', 'square', 'rectangular', uses rectangular detector
                 - 'mask' flexible detector, any 2D array
-        geometry (variable) : valid entries are determined by the `mode`, values in pixels
-            argument, as follows:
+        geometry (variable) : valid entries are determined by the `mode`, values
+            in pixels argument, as follows:
                 - 'point': 2-tuple, (qx,qy),
                    qx and qy are each single float or int to define center
                 - 'circle' or 'circular': nested 2-tuple, ((qx,qy),radius),
                    qx, qy and radius, are each single float or int
-                - 'annular' or 'annulus': nested 2-tuple, ((qx,qy),(radius_i,radius_o)),
-                   qx, qy, radius_i, and radius_o are each single float or integer
-                - 'rectangle', 'square', 'rectangular': 4-tuple, (xmin,xmax,ymin,ymax)
-                - `mask`: flexible detector, any boolean or floating point 2D array with
-                    the same shape as datacube.Qshape
-        centered (bool)     : if False (default), the origin is in the upper left corner.
-             If True, the mean measured origin in the datacube calibrations
-             is set as center. The measured origin is set with datacube.calibration.set_origin()
-             In this case, for example, a centered bright field image could be defined 
-             by geometry = ((0,0), R). For `mode="mask"`, has no effect.
-        calibrated (bool)   : if True, geometry is specified in units of 'A^-1' instead of pixels.
-            The datacube's calibrations must have its `"Q_pixel_units"` parameter set to "A^-1".
-            Setting `calibrated=True` automatically performs centering, regardless of the
-            value of the `centered` argument. For `mode="mask"`, has no effect.
-        shift_center (bool) : if True, the mask is shifted at each real space position to
-            account for any shifting of the origin of the diffraction images. The datacube's
-            calibration['origin'] parameter must be set (centered = True). The shift applied to each 
-            pattern is the difference between the local origin position and the mean origin position
-            over all patterns, rounded to the nearest integer for speed.
-        verbose (bool)      : if True, show progress bar
-        dask (bool)         : if True, use dask arrays
-        return_mask (bool)  : if False (default) returns a virtual image as usual.  If True, does
-            *not* generate or return a virtual image, instead returning the mask that would be
-            used in virtual image computation for any call to this function where
-            `shift_center = False`.  Otherwise, must be a 2-tuple of integers corresponding
-            to a scan position (rx,ry); in this case, returns the mask that would be used for
-            virtual image computation at this scan position with `shift_center` set to `True`.
+                - 'annular' or 'annulus': nested 2-tuple, ((qx,qy),
+                  (radius_i,radius_o))
+                - 'rectangle', 'square', 'rectangular': a 4-tuple,
+                  (xmin,xmax,ymin,ymax)
+                - `mask`: flexible detector, any boolean or floating point
+                  2D array with the same shape as datacube.Qshape
+        centered (bool): if False (default), the origin is in the upper left
+            corner. If True, the mean measured origin in the datacube
+            calibrations is set as center. The measured origin is set with
+            datacube.calibration.set_origin(). In this case, for example, a
+            centered bright field image could be defined by
+            geometry = ((0,0), R). For `mode="mask"`, has no effect.
+        calibrated (bool): if True, geometry is specified in units of 'A^-1'
+            instead of pixels. The datacube's calibrations must have its
+            `"Q_pixel_units"` parameter set to "A^-1". For `mode="mask"`, has
+             no effect.
+        shift_center (bool): if True, the mask is shifted at each real space
+            position to account for any shifting of the origin of the
+            diffraction images. The datacube's calibration['origin'] parameter
+            must be set (centered = True). The shift applied to each pattern is
+            the difference between the local origin position and the mean origin
+            position over all patterns, rounded to the nearest integer for speed.
+        verbose (bool): if True, show progress bar
+        dask (bool): if True, use dask arrays
+        return_mask (bool or tuple): if False (default) returns a virtual image
+            as usual. If True, does *not* generate or return a virtual image,
+            instead returning the mask that would be used in virtual image
+            computation for any call to this function where
+            `shift_center = False`. Otherwise, must be a 2-tuple of integers
+            corresponding to a scan position (rx,ry); in this case, returns the
+            mask that would be used for virtual image computation at this scan
+            position with `shift_center` set to `True`.
+        test_config: if True, returns the Boolean value of (`centered`,
+            `calibrated`,`shift_center`). Does not compute the virtual image.
 
     Returns:
-        virtual image (2D-array)
+        (2D array) virtual image
     '''
 
     assert mode in ('point', 'circle', 'circular', 'annulus', 'annular', 'rectangle', 'square', 'rectangular', 'mask'),\
     'check doc strings for supported modes'
-    g = geometry
+    if shift_center == True:
+        assert centered, "centered must be True if shift_center is True"
+    if test_config:
+        for x,y in zip(['centered','calibrated','shift_center'],
+                       [centered,calibrated,shift_center]):
+            print(f"{x} = {y}")
 
-    # Get calibration metadata
-    if centered:
-        assert datacube.calibration.get_origin(), "origin need to be calibrated"
-        x0, y0 = datacube.calibration.get_origin()
-        x0_mean = np.mean(x0)
-        y0_mean = np.mean(y0)
-    if calibrated:
-        assert datacube.calibration['Q_pixel_units'] == 'A^-1', \
-        'check datacube.calibration. datacube must be calibrated in A^-1 to use `calibrated=True`'
-
-    # Convert units into detector pixels, if `centered` or `calibrated` are True
-    if centered == True:
-        if mode == 'point':
-            g = (g[0] + x0_mean, g[1] + y0_mean)
-        if mode in('circle', 'circular', 'annulus', 'annular'):
-            g = ((g[0][0] + x0_mean, g[0][1] + y0_mean), g[1])
-        if mode in('rectangle', 'square', 'rectangular') :
-             g = (g[0] + x0_mean, g[1] + x0_mean, g[2] + y0_mean, g[3] + y0_mean)
-
-    if calibrated == True:
-        unit_conversion = datacube.calibration['Q_pixel_size']
-        if mode == 'point':
-            g = (g[0]/unit_conversion, g[1]/unit_conversion)
-        if mode in('circle', 'circular'):
-            g = ((g[0][0]/unit_conversion, g[0][1]/unit_conversion),
-                (g[1]/unit_conversion))
-        if mode in('annulus', 'annular'):
-            g = ((g[0][0]/unit_conversion, g[0][1]/unit_conversion),
-                (g[1][0]/unit_conversion, g[1][1]/unit_conversion))
-        if mode in('rectangle', 'square', 'rectangular') :
-            g = (g[0]/unit_conversion, g[1]/unit_conversion,
-                 g[2]/unit_conversion, g[3]/unit_conversion)
-
-    if shift_center == True: 
-        assert centered, "centered must be True" 
+    # Get geometry
+    g = get_calibrated_geometry(
+        datacube,
+        mode,
+        geometry,
+        centered,
+        calibrated
+    )
 
     # Get mask
     mask = make_detector(datacube.Qshape, mode, g)
@@ -111,11 +98,11 @@ def get_virtual_image(
     if return_mask == True and shift_center == False:
         return mask
 
+
     # Calculate images
 
     # no center shifting
     if shift_center == False:
-
         # dask 
         if dask == True:
 
@@ -136,7 +123,10 @@ def get_virtual_image(
         else:
 
             # compute
-            virtual_image = np.zeros(datacube.Rshape)
+            if mask.dtype == 'complex':
+                virtual_image = np.zeros(datacube.Rshape, dtype = 'complex')
+            else:
+                virtual_image = np.zeros(datacube.Rshape)
             for rx,ry in tqdmnd(
                 datacube.R_Nx,
                 datacube.R_Ny,
@@ -148,8 +138,10 @@ def get_virtual_image(
     else:
 
         # get shifts
-        qx_shift = (x0-x0_mean).round().astype(int)
-        qy_shift = (y0-y0_mean).round().astype(int)
+        assert datacube.calibration.get_origin_shift(), "origin need to be calibrated"
+        qx_shift,qy_shift = datacube.calibration.get_origin_shift()
+        qx_shift = qx_shift.round().astype(int)
+        qy_shift = qy_shift.round().astype(int)
 
         # if return_mask is True, skip computation
         if return_mask is not False:
@@ -166,7 +158,11 @@ def get_virtual_image(
             return _mask
 
         # compute
-        virtual_image = np.zeros(datacube.Rshape)
+        if mask.dtype == 'complex':
+            virtual_image = np.zeros(datacube.Rshape, dtype = 'complex')
+        else:
+            virtual_image = np.zeros(datacube.Rshape)
+
         for rx,ry in tqdmnd(
             datacube.R_Nx,
             datacube.R_Ny,
@@ -182,8 +178,87 @@ def get_virtual_image(
 
     return virtual_image
 
+
+def get_calibrated_geometry(
+    calibration,
+    mode,
+    geometry,
+    centered,
+    calibrated
+    ):
+    """
+    Determine the detector geometry in pixels, given some mode and geometry
+    in calibrated units, where the calibration state is specified by {
+    centered, calibrated}
+
+    Args:
+        calibration (Calibration, DataCube, any object with a .calibration attr,
+        or None) Used to retrieve the center positions. If `None`, confirms that
+        centered and calibrated are False then passes
+        mode: see py4DSTEM.process.virtualimage.get_virtual_image
+        geometry: see py4DSTEM.process.virtualimage.get_virtual_image
+        centered: see py4DSTEM.process.virtualimage.get_virtual_image
+        calibrated: see py4DSTEM.process.virtualimage.get_virtual_image
+
+    Returns:
+        (tuple) the geometry in detector pixels
+    """
+    # Parse inputs
+    g = geometry
+    if calibration is None:
+        assert calibrated is False and centered is False
+        return g
+    elif isinstance(calibration, Calibration):
+        cal = calibration
+    else:
+        try:
+            cal = calibration.calibration
+            assert isinstance(cal, Calibration), "`calibration.calibration` must be a Calibration instance"
+        except AttributeError:
+            raise Exception("`calibration` must either be a Calibration instance or have a .calibration attribute")
+
+    # Get calibration metadata
+    if centered:
+        assert cal.get_qx0_mean(), "origin needs to be calibrated" 
+        x0_mean, y0_mean = cal.get_origin_mean()
+
+    if calibrated:
+        assert cal['Q_pixel_units'] == 'A^-1', \
+        'check calibration - must be calibrated in A^-1 to use `calibrated=True`'
+        unit_conversion = cal.get_Q_pixel_size()
+
+
+    # Convert units into detector pixels
+
+    # Shift center
+    if centered == True:
+        if mode == 'point':
+            g = (g[0] + x0_mean, g[1] + y0_mean)
+        if mode in('circle', 'circular', 'annulus', 'annular'):
+            g = ((g[0][0] + x0_mean, g[0][1] + y0_mean), g[1])
+        if mode in('rectangle', 'square', 'rectangular') :
+             g = (g[0] + x0_mean, g[1] + x0_mean, g[2] + y0_mean, g[3] + y0_mean)
+
+    # Scale by the detector pixel size
+    if calibrated == True:
+        if mode == 'point':
+            g = (g[0]/unit_conversion, g[1]/unit_conversion)
+        if mode in('circle', 'circular'):
+            g = ((g[0][0]/unit_conversion, g[0][1]/unit_conversion),
+                (g[1]/unit_conversion))
+        if mode in('annulus', 'annular'):
+            g = ((g[0][0]/unit_conversion, g[0][1]/unit_conversion),
+                (g[1][0]/unit_conversion, g[1][1]/unit_conversion))
+        if mode in('rectangle', 'square', 'rectangular') :
+            g = (g[0]/unit_conversion, g[1]/unit_conversion,
+                 g[2]/unit_conversion, g[3]/unit_conversion)
+
+    return g
+
+
+
 def make_detector(
-    Qshape,
+    shape,
     mode,
     geometry,
 ):
@@ -191,7 +266,7 @@ def make_detector(
     Function to return 2D mask
 
     Args:
-        Qshape (tuple)     : defines shape of mask (Q_Nx, Q_Ny) where Q_Nx and Q_Ny are mask sizes
+        shape (tuple)      : defines shape of mask, for example (Q_Nx, Q_Ny) where Q_Nx and Q_Ny are mask sizes
         mode (str)         : defines geometry mode for calculating virtual image
             options:
                 - 'point' uses singular point as detector
@@ -199,7 +274,8 @@ def make_detector(
                 - 'annular' or 'annulus' uses annular detector, like dark field
                 - 'rectangle', 'square', 'rectangular', uses rectangular detector
                 - 'mask' flexible detector, any boolean or floating point 2D array with
-                    the same shape as datacube.Qshape
+                    the same shape as datacube.Qshape or datacube.Rshape for virtual image
+                    or diffraction image respectively
         geometry (variable) : valid entries are determined by the `mode`, values in pixels
             argument, as follows:
                 - 'point': 2-tuple, (qx,qy),
@@ -210,7 +286,8 @@ def make_detector(
                    qx, qy, radius_i, and radius_o are each single float or integer
                 - 'rectangle', 'square', 'rectangular': 4-tuple, (xmin,xmax,ymin,ymax)
                 - `mask`: flexible detector, any boolean or floating point 2D array with the
-                    same shape as datacube.Qshape
+                    same shape as datacube.Qshape or datacube.Rshapefor virtual image
+                    or diffraction image respectively
 
     Returns:
         virtual detector in the form of a 2D mask (array)
@@ -220,7 +297,7 @@ def make_detector(
     #point mask 
     if mode == 'point':
         assert(isinstance(g,tuple) and len(g)==2), 'specify qx and qy as tuple (qx, qy)'
-        mask = np.zeros(Qshape)
+        mask = np.zeros(shape, dtype=bool)
 
         qx = int(g[0])
         qy = int(g[1])
@@ -232,7 +309,7 @@ def make_detector(
         assert(isinstance(g,tuple) and len(g)==2 and len(g[0])==2 and isinstance(g[1],(float,int))), \
         'specify qx, qy, radius_i as ((qx, qy), radius)'
 
-        qxa, qya = np.indices(Qshape)
+        qxa, qya = np.indices(shape)
         mask = (qxa - g[0][0]) ** 2 + (qya - g[0][1]) ** 2 < g[1] ** 2
 
     #annular mask 
@@ -242,7 +319,7 @@ def make_detector(
 
         assert g[1][1] > g[1][0], "Inner radius must be smaller than outer radius"
 
-        qxa, qya = np.indices(Qshape)
+        qxa, qya = np.indices(shape)
         mask1 = (qxa - g[0][0]) ** 2 + (qya - g[0][1]) ** 2 > g[1][0] ** 2
         mask2 = (qxa - g[0][0]) ** 2 + (qya - g[0][1]) ** 2 < g[1][1] ** 2
         mask = np.logical_and(mask1, mask2)
@@ -251,7 +328,7 @@ def make_detector(
     if mode in('rectangle', 'square', 'rectangular') :
         assert(isinstance(g,tuple) and len(g)==4), \
        'specify x_min, x_max, y_min, y_max as (x_min, x_max, y_min, y_max)'
-        mask = np.zeros(Qshape)
+        mask = np.zeros(shape, dtype=bool)
 
         xmin = int(np.round(g[0]))
         xmax = int(np.round(g[1]))
@@ -263,10 +340,84 @@ def make_detector(
     #flexible mask
     if mode == 'mask':
         assert type(g) == np.ndarray, '`geometry` type should be `np.ndarray`'
-        assert (g.shape == Qshape), 'mask and diffraction pattern shapes do not match'
+        assert (g.shape == shape), 'mask and diffraction pattern shapes do not match'
         mask = g
-
     return mask
+    
+def make_bragg_mask(
+    Qshape,
+    g1,
+    g2,
+    radius,
+    origin,
+    max_q,
+    return_sum = True,
+    **kwargs,
+    ):
+    '''
+    Creates and returns a mask consisting of circular disks
+    about the points of a 2D lattice.
+
+    Args:
+        Qshape (2 tuple): the shape of diffraction space
+        g1,g2 (len 2 array or tuple): the lattice vectors
+        radius (number): the disk radius
+        origin (len 2 array or tuple): the origin
+        max_q (nuumber): the maxima distance to tile to
+        return_sum (bool): if False, return a 3D array, where each
+            slice contains a single disk; if False, return a single
+            2D masks of all disks
+
+    Returns:
+        (2 or 3D array) the mask
+    '''
+    nas = np.asarray
+    g1,g2,origin = nas(g1),nas(g2),nas(origin)
+
+    # Get N,M, the maximum indices to tile out to
+    L1 = np.sqrt(np.sum(g1**2))
+    H = int(max_q/L1) + 1
+    L2 = np.hypot(-g2[0]*g1[1],g2[1]*g1[0])/np.sqrt(np.sum(g1**2))
+    K = int(max_q/L2) + 1
+
+    # Compute number of points
+    N = 0
+    for h in range(-H,H+1):
+        for k in range(-K,K+1):
+            v = h*g1 + k*g2
+            if np.sqrt(v.dot(v)) < max_q:
+                N += 1
+
+    #create mask
+    mask = np.zeros((Qshape[0], Qshape[1], N), dtype=bool)
+    N = 0
+    for h in range(-H,H+1):
+        for k in range(-K,K+1):
+            v = h*g1 + k*g2
+            if np.sqrt(v.dot(v)) < max_q:
+                center = origin + v
+                mask[:,:,N] = make_detector(
+                    Qshape,
+                    mode = 'circle',
+                    geometry = (center, radius),
+                )
+                N += 1
+
+
+    if return_sum:
+        mask = np.sum(mask, axis = 2)
+    return mask
+
+
+
+
+
+
+
+
+
+
+
 
 
 def get_virtual_image_pointlistarray(
